#!/usr/bin/env python

###
# Copyright (c) 2002, Jeremiah Fincher
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
###

"""
Module for general worldly stuff, like global variables and whatnot.
"""

__revision__ = "$Id$"

import fix

import gc
import os
import sre
import time
import types
import atexit
import threading

import log
import conf

startedAt = 0.0

mainThread = threading.currentThread()

threadsSpawned = 1 # Starts at one for the initial "thread."
commandsProcessed = 0
###
# End Global Values.
###

ircs = []

flushers = [] # A periodic function will flush all these.

def flush():
    """Flushes all the registered flushers."""
    for f in flushers:
        f()

tempvars = {} # A storage place for temporary variables that need to be
              # globally accessible.


def upkeep(): # Function to be run on occasion to do upkeep stuff.
    """Does upkeep (like flushing, garbage collection, etc.)"""
    collected = gc.collect()
    if os.name == 'nt':
        try:
            import msvcrt
            msvcrt.heapmin()
        except ImportError:
            pass
        except IOError: # Win98 sux0rs!
            pass
    if gc.garbage:
        log.warning('Uncollectable garbage: %s', gc.garbage)
    if 'noflush' not in tempvars:
        flush()
    log.verbose('Regexp cache size: %s', len(sre._cache))
    log.info('%s upkeep ran.', time.strftime(conf.logTimestampFormat))
    return collected

def makeIrcsDie():
    for irc in ircs[:]:
        irc.die()

def startDying():
    global dying
    dying = True

atexit.register(upkeep)
atexit.register(makeIrcsDie)
atexit.register(startDying)

##################################################
##################################################
##################################################
## Don't even *think* about messing with these. ##
##################################################
##################################################
##################################################
startup = False
testing = False
dying = False

# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:
